#!/usr/bin/perl

require "config.pl";

#Check for LOCK-File:
if (open (LOCK,"<$_lockfile")){
  $msg = <LOCK>;
  print STDERR "indexer is already running with PID ".$msg."\n";
  print STDERR "if not remove lockfile '$_lockfile'\n";
  exit(-1);
}else{
  open (LOCK,">$_lockfile") or die ("Could not create lockfile");
  print LOCK $$;
  close (LOCK);
}

debug("using pattern '$_pattern'");

# Get servers to reindex
$dbh = newdbh();
$result = $dbh->selectall_arrayref("SELECT ServerName, User, Password, ServerID, Port FROM Server
                                     WHERE LastConnect IS NULL
		                        OR LastConnect < DATE_SUB( NOW(), INTERVAL $_reindex DAY )");

$starttime = time();

$row=0;
$running=0;
while (defined($result->[$row][0])){
  if($running >= $_maxprocs){
    wait();
    $running--;
  }else{
    if (fork()==0){
      $mydbh = newdbh();
      #Set up ftpls params
      $param="-R -m --tries=$_retries";            #General Options
      $param=$param." -u ".$result->[$row][1];     #Username
      $param=$param." -p ".$result->[$row][2];     #Password
      $param=$param." ".$result->[$row][0];;       #Server
      $param=$param.":".$result->[$row][4]." /";   #Port and Root-DIR
      quotemeta($param);

      if (!$_debug){
        #Remove ftpls errormessages
        $param=$param." 2>/dev/null";
      }
     
      $server=$result->[$row][3];                #ServerID
      
      #Run ftpls
      debug("now scanning ".$result->[$row][0]." (ID: $server)");
      @filelist=undef;
      @filelist=`$_ftpls $param`;
      $exit = ($? >> 8);# $?; #/25600;
      debug("ftpls returned with status $exit");
      
      debug("found ".int(scalar(@filelist)/4)." files");

      #Delete all previous files (no update supported
      $mydbh->do("DELETE FROM Files WHERE ServerID = $server") or die "del files failed: ".$mydbh->errstr;
      
      if ( scalar(@filelist)<4 ){ #ftpls reurns four lines for each for each file
        #Execution failed -> remove Server
        $mydbh->do("DELETE FROM Server WHERE ServerID = $server") or die "del server failed: ".$mydbh->errstr;
        debug ("Server with ID $server deleted");
      }else{
        #everything was fine -> add files to database
        $added=0; #Reset Filecounter;
        while (scalar(@filelist)>3){
         
          #get infos from array
          do {
            $N = shift(@filelist);
          }while( (substr($N,0,1) ne 'N') || !defined($N) );
          $K = shift(@filelist);
          $S = shift(@filelist);
          $T = shift(@filelist);
          
          #remove first letter
          $N = substr($N,1);
          $K = substr($K,1); 
          $S = substr($S,1);
          $T = substr($T,1);
          
          #remove trailing newline (not really needed)
          chomp($N);
          chomp($K);
          chomp($S);
          chomp($T);
          
          #remove trailing whitespaces (not really needed)
          $N =~ s/ *$//;
          $K =~ s/ *$//;
          $S =~ s/ *$//;
          $T =~ s/ *$//;
          
          #Skip directories
          if ($K eq "dir") {next;}
          #Skip nothing :-)
          if ($N eq "") {next;}
          #Skip unwanted files:
          if (defined($_pattern)){
            if (!($N =~ m/$_pattern/i)) {
    	        debug("skipping  '$N' (no patternmatch)");
    	        next;
    	      }
          }
          #Skip files smaller than 1 meg
          if ($S < $_minsize){
            debug("skipping  '$N' (too small)");
    	      next;
          }
    	 
          #Insert in database;
          debug("inserting '$N' into DB");
          $N = quotemeta($N);
          $K = quotemeta($K);
          $S = quotemeta($S);
          $T = quotemeta($T);
          $mydbh->do("INSERT INTO Files (ServerID, FileName, Date, Size)
                 VALUES ($server,'$N','$T',$S)") or die "ins files failed: ".$mydbh->errstr;
          $added++; #Increase Filecounter;
        }
        debug ("$added files were added");
        #We only keep servers with interessting files:
        if ($added){
          $mydbh->do("UPDATE Server SET LastConnect=NOW() WHERE ServerID=$server") or die "upd server failed: ".$mydbh-errstr;
        }else{
          $mydbh->do("DELETE FROM Server WHERE ServerID = $server") or die "del server failed: ".$mydbh->errstr;
          debug ("Server with ID $server deleted");
        }
      }
      $mydbh->disconnect;
      exit();
    }else{
      $running++;
    }
  }
  $row++;
}

#Wait for forgotten childs
while (wait() != -1){};

$neededtime = sprintf( "%.2f",(time()-$starttime)/60 );
debug("FINISHED: $row servers scanned in $neededtime minutes");
$dbh->disconnect;

unlink ($_lockfile);
